from typing import Optional

from bot_api.abc.dto_abc import DtoABC
from bot_data.model.server import Server


class UserDTO(DtoABC):
    def __init__(
        self,
        id: int = None,
        dc_id: int = None,
        xp: int = None,
        server: Optional[Server] = None,
        is_technician: Optional[bool] = None,
        is_admin: Optional[bool] = None,
        is_moderator: Optional[bool] = None,
    ):
        DtoABC.__init__(self)

        self._user_id = id
        self._discord_id = dc_id
        self._xp = xp
        self._server = server

        self._is_technician = is_technician
        self._is_admin = is_admin
        self._is_moderator = is_moderator

    @property
    def user_id(self) -> int:
        return self._user_id

    @property
    def discord_id(self) -> int:
        return self._discord_id

    @property
    def xp(self) -> int:
        return self._xp

    @xp.setter
    def xp(self, value: int):
        self._xp = value

    @property
    def server(self) -> Optional[Server]:
        return self._server

    @property
    def is_technician(self) -> bool:
        return self._is_technician if self._is_technician is not None else False

    @property
    def is_admin(self) -> bool:
        return self._is_admin if self._is_admin is not None else False

    @property
    def is_moderator(self) -> bool:
        return self._is_moderator if self._is_moderator is not None else False

    def from_dict(self, values: dict):
        self._user_id = values["id"]
        self._discord_id = values["dcId"]
        self._xp = values["xp"]
        self._server = values["server"]

    def to_dict(self) -> dict:
        return {
            "id": self._user_id,
            "dcId": self._discord_id,
            "xp": self._xp,
            "server": self._server.id,
            "isTechnician": self.is_technician,
            "isAdmin": self.is_admin,
            "isModerator": self.is_moderator,
        }
