from abc import ABC, abstractmethod
from datetime import datetime
from typing import Callable, Union

import discord
from cpl_query.extension import List
from discord.ext.commands import Context

from bot_data.model.auto_role_rule import AutoRoleRule
from bot_data.model.server_config import ServerConfig
from bot_data.model.user import User


class ClientUtilsABC(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def received_command(self, guild_id: int):
        pass

    @abstractmethod
    def moved_user(self, guild_id: int):
        pass

    @abstractmethod
    def moved_users(self, guild_id: int, count: int):
        pass

    @abstractmethod
    def get_client(self, dc_ic: int, guild_id: int):
        pass

    @abstractmethod
    async def check_if_bot_is_ready_yet(self) -> bool:
        pass

    @abstractmethod
    async def check_if_bot_is_ready_yet_and_respond(self, ctx: Context) -> bool:
        pass

    @abstractmethod
    async def presence_game(self, t_key: str):
        pass

    @abstractmethod
    def get_auto_complete_list(self, _l: List, current: str, select: Callable = None) -> List:
        pass

    @abstractmethod
    def is_message_xp_count_by_hour_higher_that_max_message_count_per_hour(
        self,
        created_at: datetime,
        user: User,
        settings: ServerConfig,
        is_reaction: bool = False,
    ) -> bool:
        pass

    @abstractmethod
    def get_ontime_for_user(self, user: User) -> float:
        pass

    @abstractmethod
    async def react_to_message_by_auto_role_rule(
        self,
        discord_channel_id: int,
        discord_message_id: int,
        rule: AutoRoleRule,
        guild: discord.Guild,
    ):
        pass

    @abstractmethod
    async def check_default_role(self, member: Union[discord.User, discord.Member]):
        pass
