from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC

from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.server_config_repository_abc import ServerConfigRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.server import Server
from bot_data.model.technician_config import TechnicianConfig
from bot_data.service.server_config_seeder import ServerConfigSeeder


class ConfigService:
    def __init__(
        self,
        config: ConfigurationABC,
        services: ServiceProviderABC,
        technician_config_repo: TechnicianConfigRepositoryABC,
        server_config_repo: ServerConfigRepositoryABC,
        server_seeder: ServerConfigSeeder,
    ):
        self._config = config
        self._services = services
        self._technician_config_repo = technician_config_repo
        self._server_config_repo = server_config_repo

        self._server_seeder = server_seeder

    def reload_technician_config(self):
        technician_config = self._technician_config_repo.get_technician_config()
        self._config.add_configuration(TechnicianConfig, technician_config)
        self._config.add_configuration(
            FeatureFlagsSettings,
            FeatureFlagsSettings(**technician_config.feature_flags),
        )

    async def reload_server_config(self, server: Server):
        if not self._server_config_repo.does_server_config_exists(server.id):
            await self._server_seeder.seed()

        server_config = self._server_config_repo.get_server_config_by_server(server.id)
        self._config.add_configuration(
            f"{type(server_config).__name__}_{server_config.server.discord_id}",
            server_config,
        )
