import { Component, OnDestroy, OnInit } from "@angular/core";
import { TranslateService } from "@ngx-translate/core";
import { MenuItem } from "primeng/api";
import { AuthService } from "src/app/services/auth/auth.service";
import { ThemeService } from "src/app/services/theme/theme.service";
import { SidebarService } from "../../services/sidebar/sidebar.service";
import { Subject } from "rxjs";
import { takeUntil } from "rxjs/operators";

@Component({
  selector: "app-sidebar",
  templateUrl: "./sidebar.component.html",
  styleUrls: ["./sidebar.component.scss"]
})
export class SidebarComponent implements OnInit, OnDestroy {

  menuItems: MenuItem[]= [];
  private unsubscriber = new Subject<void>();

  constructor(
    private authService: AuthService,
    private translateService: TranslateService,
    private themeService: ThemeService,
    private sidebar: SidebarService
  ) {
    this.sidebar.menuItems$.pipe(
      takeUntil(this.unsubscriber)
    ).subscribe(value => {
      this.menuItems = value;
    });
  }

  ngOnInit(): void {
    this.themeService.loadMenu();
  }

  ngOnDestroy() {
    this.unsubscriber.next();
    this.unsubscriber.complete();
  }

}
