import { HttpClient } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { SettingsService } from "../settings/settings.service";
import { map, Observable } from "rxjs";
import { Variables } from "../../models/graphql/variables.model";
import { ActivatedRoute, Router } from "@angular/router";
import { Server } from "../../models/data/server.model";
import { Queries } from "../../models/graphql/queries.model";
import { Query } from "../../models/graphql/query.model";
import { SpinnerService } from "../spinner/spinner.service";
import { GraphQLResult } from "../../models/graphql/result.model";
import { ToastService } from "../toast/toast.service";
import { TranslateService } from "@ngx-translate/core";
import { ServerService } from "../server.service";

@Injectable({
  providedIn: "root"
})
export class DataService {

  constructor(
    private appsettings: SettingsService,
    private http: HttpClient,
    private server: ServerService,
    private spinner: SpinnerService,
    private router: Router,
    private toast: ToastService,
    private translate: TranslateService,
  ) {
  }

  public getServerFromRoute(route: ActivatedRoute): Promise<Server> {
    return new Promise((resolve, reject) => {
      this.spinner.showSpinner();
      if (!route.snapshot.params["serverId"] || route.snapshot.params["serverId"] == "undefined") {
        this.spinner.hideSpinner();
        this.router.navigate(["/dashboard"]);
        reject();
      }

      this.query<Server>(Queries.serversQuery, {
          filter: { id: route.snapshot.params["serverId"] }
        },
        function(data: Query) {
          return data.servers.length > 0 ? data.servers[0] : null;
        }
      ).subscribe(server => {
        this.server.setServer(server);
        this.spinner.hideSpinner();
        resolve(server);
      });
    });

  }

  public query<T>(query: string, variables?: Variables, f?: Function): Observable<T> {
    return this.http
      .post<GraphQLResult>(`${this.appsettings.getApiURL()}/api/graphql`, {
        query: query,
        variables: variables
      })
      .pipe(map(d => {
        if (d.errors && d.errors.length > 0) {
          throw new Error(d.errors.map((x: { message: String }) => x.message).toString());
        }
        return d.data;
      }))
      .pipe(map((d) => f ? f(d) : d));
  }

  public mutation<T>(query: string, variables?: object, f?: Function): Observable<T> {
    return this.http
      .post<GraphQLResult>(`${this.appsettings.getApiURL()}/api/graphql`, {
        query: query,
        variables: variables
      })
      .pipe(map(d => {
        if (d.errors && d.errors.length > 0) {
          d.errors.forEach((error: Error) => {
            this.toast.error(this.translate.instant("common.error"), error.message)
          });

          throw new Error(d.errors.toString());
        }
        return d.data;
      }))
      .pipe(map((d) => f ? f(d) : d));
  }

}
